<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Repositories;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\CountryAndLanguage;
use Locr\WordPress\Plugin\LocrMaps\TypedSequence;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icons.
 */
final class CountriesAndLanguages {
	/**
	 * This function gets all countries and languages.
	 *
	 * @var ?TypedSequence
	 */
	private static ?TypedSequence $countries_and_languages = null;

	/**
	 * This function gets all icons.
	 *
	 * @return TypedSequence The countries and languages.
	 * @throws \Exception If the countries and languages file does not exist.
	 */
	public function get_all(): TypedSequence {
		if ( is_null( self::$countries_and_languages ) ) {
			self::$countries_and_languages = TypedSequence::for_type( CountryAndLanguage::class );

			$json_filename = plugin_dir_path( dirname( dirname( __DIR__ ) ) ) . 'data/countries_and_languages.json';
			if ( ! file_exists( $json_filename ) ) {
				throw new \Exception( 'The countries and languages file does not exist.' );
			}

			global $wp_filesystem;
			// @phpstan-ignore requireOnce.fileNotFound
			require_once ABSPATH . 'wp-admin/includes/file.php';
			WP_Filesystem();
			// @phpstan-ignore method.nonObject
			$json_content = $wp_filesystem->get_contents( $json_filename );
			if ( ! is_string( $json_content ) ) {
				throw new \Exception( 'The countries and languages file could not be read.' );
			}
			$json_data = json_decode( $json_content, true );
			if ( is_null( $json_data ) ) {
				throw new \Exception( 'The countries and languages file could not be decoded.' );
			}
			if ( ! is_array( $json_data ) ) {
				throw new \Exception( 'The countries and languages data is not an array.' );
			}

			foreach ( $json_data as $row ) {
				if ( is_array( $row ) ) {
					self::$countries_and_languages->add( CountryAndLanguage::from_array( $row ) );
				}
			}
		}

		return self::$countries_and_languages;
	}
}
